<?php

/*
 * Prevent Direct Access
 */
defined( 'ABSPATH' ) or die( "Sorry, Can not access page directly." );

/**
 * Class for Autovid Profit Pro DailyMotion support
 */
if ( ! class_exists( 'Autovid_Profit_DailyMotion' )) {
    class Autovid_Profit_DailyMotion {

	private $vbp = null;			// will point to main AutoVid_Profit instance
        private $query_fields = array();        // options for current query

	private $APIs = array(
		'channel.data'		=> 'https://api.dailymotion.com/channel/',
		'group.data'		=> 'https://api.dailymotion.com/group/',
		'playlist.data'		=> 'https://api.dailymotion.com/playlist/',
		'user.data'		=> 'https://api.dailymotion.com/user/',
		'videos.list'		=> 'https://api.dailymotion.com/videos',
		'video.data'		=> 'https://api.dailymotion.com/video/'
		);

	private $batch_limit = 100;		// max amount we can request per query

	/**
	 * Create DailyMotion video source
	 * Point back to Autovid Profit Pro object to use common functions
	 * Save the query fields for easy access
	 */
	public function __construct( $vbp, $query_fields ) {
		$this->vbp = $vbp;
		$this->query_fields = $query_fields;
	}

	/**
	 * Shortcut to proper API url
	 */
	private function getApi( $type ) {
		return $this->APIs[$type];
	}

	/**
	 * Make the query and check for errors.
	 */
	private function queryApi( $url ) {
		$this->vbp->info_message( __FUNCTION__ . ': ' . htmlentities( $url ), 'updated', 'debug' );

		$response = wp_remote_get( $url );
		if ( is_wp_error( $response ) ) {
			return $this->vbp->info_message( sprintf( __( 'WP error %s in %s(%s) - %s ', 'autovid-profit' ), $response->get_error_code(), __FUNCTION__, $url, $response->get_error_message() ) );
		}

		$data = json_decode( wp_remote_retrieve_body( $response ) );

		if ( isset( $data->error ) ) {
			$this->vbp->info_message( sprintf( __( '%s(%s) - DailyMotion API returned %s: %s', 'autovid-profit' ), __FUNCTION__, $url, $data->error->code, $data->error->message ) );
		}

		return $data;
	}

	private function get_best_thumbnail( $video ) {
		$img = array();
		$match = false;

		if ( ! $match && isset( $video->thumbnail_60_url ) ) $img[] = (string) $video->thumbnail_60_url;
		if ( $this->query_fields['qExtraParams2'] == '60px' ) $match = true;

		if ( ! $match && isset( $video->thumbnail_120_url ) ) $img[] = (string) $video->thumbnail_120_url;
		if ( $this->query_fields['qExtraParams2'] == '120px' ) $match = true;

		if ( ! $match  && isset( $video->thumbnail_180_url ) ) $img[] = (string) $video->thumbnail_180_url;
		if ( $this->query_fields['qExtraParams2'] == '180px' ) $match = true;

		if ( ! $match && isset( $video->thumbnail_240_url ) ) $img[] = (string) $video->thumbnail_240_url;
		if ( $this->query_fields['qExtraParams2'] == '240px' ) $match = true;

		if ( ! $match && isset( $video->thumbnail_360_url ) ) $img[] = (string) $video->thumbnail_360_url;
		if ( $this->query_fields['qExtraParams2'] == '360px' ) $match = true;

		if ( ! $match && isset( $video->thumbnail_480_url ) ) $img[] = (string) $video->thumbnail_480_url;
		if ( $this->query_fields['qExtraParams2'] == '480px' ) $match = true;

		if ( ! $match && isset( $video->thumbnail_720_url ) ) $img[] = (string) $video->thumbnail_720_url;
		if ( $this->query_fields['qExtraParams2'] == '720px' ) $match = true;

		// catchall
		if ( isset( $video->thumbnail_url ) ) $img[] = (string) $video->thumbnail_url;

		$img = array_reverse( $img );

		$image = $this->vbp->verify_thumbnails( $img );

		return $image;
	}

	/**
	 * Extract video details into our generic videoInfo array
	 */
	private function get_video_info( $video ) {
		$videoInfo = array();

		$videoInfo['videoID'] = isset( $video->id ) ? (string) $video->id : '';
		$videoInfo['url'] = isset( $video->embed_url ) ? $video->embed_url : '';
		$videoInfo['videoEmbed'] = isset( $video->embed_html ) ? $video->embed_html : '';
		$videoInfo['orig_title'] = $videoInfo['title'] = isset( $video->title ) ? (string) $video->title : '';
		$videoInfo['orig_desc'] = $videoInfo['desc'] = isset( $video->description ) ? (string) $video->description : '';
		$videoInfo['association'] = isset( $video->{"owner.username"}) ? (string) $video->{"owner.username"} : '';
		$videoInfo['img'] = $this->get_best_thumbnail( $video );

		$videoInfo['allow_embed'] = isset( $video->allow_embed ) ? (string) $video->allow_embed : 'true';

		$publishedAt = isset( $video->updated_time ) ? (string) $video->updated_time : current_time( 'Y-m-d H:i:s' );
		// video time is UTC - convert to date string
		$publishedAt = date( 'Y-m-d H:i:s', $publishedAt );
		$date = $this->vbp->getDateTime( $publishedAt, $videoInfo['url'] );
		$videoInfo['publishedAt'] = isset( $date ) ? $date->format( 'Y-m-d H:i:s' ) : $publishedAt;
		$videoInfo['publishedAt8601'] = isset( $date ) ? $date->format( 'c' ) : '';

		$videoInfo['duration'] = isset( $video->duration_formatted ) ? $video->duration_formatted : 0;

		$videoInfo['sourceCategory'] = $videoInfo['tags'] = '';
		if ( TRUE == $this->query_fields['cAddSourceCategory'] && isset( $video->channel ) ) {
			$videoInfo['sourceCategoryName'] = (string) $video->channel;
			$videoInfo['sourceCategory'] = $this->vbp->create_the_category( (string) $video->channel );
		}
		if ( TRUE == $this->query_fields['pGrabTags'] && isset( $video->tags ) ) {
			$videoInfo['tags'] = implode( ',', $video->tags );
		}

		$videoInfo['viewCount'] = isset( $video->views_total) ? (string) $video->views_total : 0;
		//convert DailyMotion ratings to likes/dislikes
		$rating = isset( $video->rating ) ? (string) $video->rating : 0;
		$ratings_total = isset( $video->ratings_total ) ? (string) $video->ratings_total : 0;
		$videoInfo['likeCount'] = round( $ratings_total * ( $rating * 0.2 ), 0) ; // 1-5 rating system
		$videoInfo['dislikeCount'] = $ratings_total - $videoInfo['likeCount'];

		$videoInfo['videoSource'] = 'DailyMotion';

		$videoInfo = apply_filters( 'get_video_info', $videoInfo );

		return $videoInfo;
	}

	/**
	 * Do a batch query search for up to $this->batch_limit comments at a time (DailyMotion limit)
	 */
	private function grab_comments_batch( $url, $postID ) {
		$this->vbp->info_message(__FUNCTION__ . ': ' . htmlentities( $url ), 'updated', 'debug' );
        	$response = $this->queryApi( $url );
		$total=0;
		foreach ( $response->list as $comment ) {
			$author = isset( $comment->{"owner.screenname"} ) ? trim( $comment->{"owner.screenname"} ) : '';
			$content = isset( $comment->message ) ? trim( $comment->message ) : '';
			if ( empty( $author ) || empty( $content ) ) {
				continue;
			}
			if ( TRUE == $this->query_fields['cUsePublishedDate'] ) {
				$comment_date = isset( $comment->created_time ) ? $comment->created_time : time();
				$comment_date = date( 'Y-m-d H:i:s', $comment_date );
			}
			if ( TRUE == $this->query_fields['pRemoveCommUrls'] ) {
				$content = $this->vbp->remove_urls( $content );
			}
			if ( TRUE == $this->query_fields['pLinkifyCommUrls'] ) {
				$content = $this->vbp->linkify_filtered( $content );
			}

			$commentdata = array(
				'comment_post_ID'	=> $postID,
				'comment_author'	=> $author,
				'comment_author_email'	=> '',
				'comment_author_url'	=> '',
				'comment_content'	=> $content,
				'comment_type'		=> '',
				'comment_date'     	=> ( TRUE == $this->query_fields['cUsePublishedDate'] ) ? $comment_date : current_time( 'Y-m-d H:i:s' ),
				'comment_approved'	=> 1
			);
			$commentdata = apply_filters( 'dailymotion_comment', $commentdata );
			if ( empty( $commentdata ) ) {
				continue;
			}
			$comment_id = wp_insert_comment( $commentdata );
			if ( $comment_id ) {
				$total++;
			}
		}
		$this->vbp->info_message( sprintf( __( '%s comments imported successfully to post id %s.', 'autovid-profit' ), $total, $postID ), 'updated', 'video_import' );

		return isset( $response->has_more ) ? $response->has_more : 0;
	}

	/**
	 * Grab and save comments for a video
	 * Toggle comment approval and email notification if set
	 * Break up process into batches
	 */
	private function grab_comments( $videoID, $postID ) {
		
       		$cm = get_option( 'comment_moderation' );
       		$cn = get_option( 'comment_notification' );
		// temporarily turn off manual approval and email notification
       		if ( 1 == $cm ) {
			update_option( 'comment_moderation', FALSE );
		}
       		if ( 1 == $cn ) {
			update_option( 'comment_notification', FALSE );
		}

		$totalComments = $this->query_fields['qNumComments'];
        	$query_args = array(
			'fields'	=> 'owner.screenname,message,created_time',
                	'limit'		=> $totalComments > $this->batch_limit || $totalComments < 0 ? $this->batch_limit : $totalComments,
                	'page'		=> '1'
        	);

		$query_args = apply_filters( 'dailymotion_comments_query', $query_args, $this->query_fields );

		while ( $totalComments ) {
			$url = $this->getApi( 'video.data' ) . $videoID . '/comments' . '?' . http_build_query( $query_args );
			if ( ! $this->grab_comments_batch( $url, $postID ) ) {
				break;	// no more comments
			}
			$totalComments -= $query_args['limit'];
                	$query_args['limit'] = $totalComments > $this->batch_limit || $totalComments < 0 ? $this->batch_limit : $totalComments;
                	$query_args['page']++;
		}

		// turn back on manual approval and email notifications, if they were set set
       		if ( 1 == $cm ) {
			update_option( 'comment_moderation', $cm );
		}
       		if ( 1 == $cn ) {
			update_option( 'comment_notification', $cn );
		}
	}

	/**
	 * Takes an array of video details to process and create posts
	 */
	private function save_videos( $videoDetails ) {
                if ( ! $videoDetails ) {
			return $this->vbp->info_message( sprintf( __( 'Error: %s - no data to save.', 'autovid-profit' ), __FUNCTION__ ) );
                }
		foreach ( $videoDetails as $video ) {
			$videoInfo = $this->get_video_info( $video );
			$this->vbp->info_message( sprintf( __( 'Checking Daily Motion video: [%s] at %s', 'autovid-profit' ), $videoInfo['title'], $videoInfo['url'] ), 'updated', 'video_import' );

			// DailyMotion specific:
			if ( $videoInfo['allow_embed'] == FALSE ) {
				$this->vbp->info_message( __( 'Video flag allow_embed is set to false. Skipping.', 'autovid-profit' ),  'updated', 'video_import' );
				continue;
			}

			$postID = $this->vbp->save_the_video( $this->query_fields, $videoInfo );

			if ( ! $postID ) continue;

			if ( ! empty( $this->query_fields['qNumComments'] ) ) {
				$this->grab_comments( $videoInfo['videoID'], $postID );
			}

			$this->vbp->publish_the_video( $postID, $this->query_fields, $videoInfo );
		}
	}

	/**
	 * Grab a single video by the video ID stored in qAssoc.
	 */
	private function grab_single_video() {
		$fields = 'id,title,channel,thumbnail_60_url,thumbnail_120_url,thumbnail_180_url,thumbnail_240_url,thumbnail_360_url,thumbnail_480_url,thumbnail_720_url,thumbnail_url,allow_embed,updated_time,description,duration_formatted,embed_url,embed_html,rating,ratings_total,status,tags,views_total,owner.username';
		$query_args = array(
			'fields'		=> $fields
		);

		$url = $this->getApi( 'video.data' ) . $this->query_fields['qAssoc'] . '?' . http_build_query( $query_args );
		$response = $this->queryApi( $url );
		if ( ! isset( $response->error ) ) {
			$this->save_videos( array( $response ) );
		}
	}


	/**
	 * Make a batch request for videos
	 */
	private function grab_videos_batch( $url ) {
		$response = $this->queryApi( $url );
		if ( ! empty( $response->list ) ) {
			$this->save_videos( $response->list );
		}
		else {
			return $this->vbp->info_message( __( 'No results returned for given query.', 'autovid-profit' ), 'updated', 'critical' );
		}
		return $response->has_more;
	}

	/**
	 * Process the video query and break down into batches
	 * http://developer.dailymotion.com/tools
	 */
	public function grab_videos() {

		$keyphrase = $this->query_fields['qKeyphrase'];
		$assoc = $this->query_fields['qAssoc'];
		$assocType = $this->query_fields['qAssocType'];
		$base_url = $this->getApi( 'videos.list' );

                if ( empty( $keyphrase ) && empty( $assoc ) && empty( $assocType ) ) {
                        return $this->vbp->info_message( __( 'Error: No DailyMotion keyphrase or association not set properly.', 'autovid-profit' ), 'error', 'critical' );
                }

		$fields = 'id,title,channel,thumbnail_60_url,thumbnail_120_url,thumbnail_180_url,thumbnail_240_url,thumbnail_360_url,thumbnail_480_url,thumbnail_720_url,thumbnail_url,allow_embed,updated_time,description,duration_formatted,embed_url,embed_html,rating,ratings_total,status,tags,views_total,owner.username';

		$totalVids = $this->query_fields['qNumVideos'];

		$query_args = array(
			'sort'			=> $this->query_fields['qOrderBy'],
			'limit'			=> $totalVids > $this->batch_limit ? $this->batch_limit : $totalVids,
			'page'			=> '1',
			'fields'		=> $fields
		);

		if ( $keyphrase ) {
			$query_args['search'] = $keyphrase;
		}

		if ( $assoc && $assocType == 'channel' ) {
			$query_args['channel'] = $assoc;
		}
		else if ( $assoc && $assocType == 'group' ) {
			$base_url = $this->getApi( 'group.data' ) . $assoc . '/videos';
		}
		else if ( $assoc && $assocType == 'playlist' ) {
			$base_url = $this->getApi( 'playlist.data' ) . $assoc . '/videos';
		}
		else if ( $assoc && $assocType == 'user' ) {
			$query_args['owners'] = $assoc;
		}
		else if ( $assoc && $assocType == 'video' ) {
			return $this->grab_single_video();
		}

		if ( $this->query_fields['qRegionCode'] ) {
			$query_args['country'] = $this->query_fields['qRegionCode'];
		}
		if ( $this->query_fields['qLanguage'] ) {
			$query_args['languages'] = $this->query_fields['qLanguage'];
		}

		if ( $this->query_fields['qPublishedAfter'] ) {
			$date = $this->vbp->getDateTime( $this->query_fields['qPublishedAfter'], $videoInfo['url'], false, false );
			if ( $date ) {
				$query_args['created_after'] = $date->format('U');
			}
			else {
				$this->vbp->info_message( sprintf( __( 'Error: publishedAfter date %s not recognized.', 'autovid-profit' ), $query_args['publishedAfter'] ) );
			}
		}
		if ( $this->query_fields['qPublishedBefore'] ) {
			$date = $this->vbp->getDateTime( $this->query_fields['qPublishedBefore'], $videoInfo['url'], false, false );
			if ( $date ) {
				$query_args['publishedBefore'] = $date->format( 'U' );
			}
			else {
				$this->vbp->info_message( sprintf( __( 'Error: publishedBefore date %s not recognized.', 'autovid-profit' ), $query_args['publishedBefore'] ) );
			}
		}

		$query_args = apply_filters( 'dailymotion_search_query', $query_args, $this->query_fields );

		while ( $totalVids > 0 ) {
			$url = $base_url . '?' . http_build_query( $query_args );
			if ( ! $this->grab_videos_batch( $url ) ) {
				break;
			}
			$totalVids -= $query_args['limit'];
			$query_argsp['limit'] = $totalVids > $this->batch_limit ? $this->batch_limit : $totalVids;
			$query_args['page']++;
		}
	}


	/**
	 * Checks a batch of video ids to see if they're been deleted from DailyMotion
	 * Query result list without id means that video has been deleted.
	 */
	private function check_for_deleted_batch( $videoIDs ) {
		if ( empty( $videoIDs ) ) {
			return 0;
		}

		foreach ( $videoIDs as $key => $value ) {
			$list[] = $value;
		}
		$commaList = implode( ',', $list );

		$this->vbp->info_message( sprintf( __( 'Schedule Checker: checking DailyMotion video IDs: %s ', 'autovid-profit' ), $commaList ), 'updated', 'utility_funcs' );

        	$query_args = array(
			'fields'	=> 'id,status',
                	'ids'		=> $commaList,
			'limit'		=> $this->batch_limit
        	);
		$url = $this->getApi( 'videos.list' ) . '?' . http_build_query( $query_args );
		$data = $this->queryApi( $url );

		if ( ! $data ) { // bad error occurred. Try again another time
			return $this->vbp->info_message( __( 'Query Error: Skipping this batch.', 'autovid-profit' ) );
		}
		if ( isset( $data->error ) ) { // error occurred - probably timeout. Try again another time
			return $this->vbp->info_message( sprintf( __( 'DailyMotion error: %s ', 'autovid-profit' ), $data->error ) );
		}

		$videoDetails = isset( $data->list ) ? $data->list: array();

		$validIDs = array();
		// remove each ID found from query from our original list of IDs to check...
		foreach ( $videoDetails as $video )  {
			if ( ( $key = array_search( $video->id, $videoIDs ) ) !== FALSE && ( $video->status == 'ready' || $video->status == 'published' ) ) {
				unset( $videoIDs[$key] );
			}
		}

		// any remaining IDs in our list means they weren't found on DailyMotion
		foreach ( $videoIDs as $key => $value ) {
			$key = apply_filters( 'vb_delete_video', $key );
			if ( ! $key ) {
				continue;
			}
			wp_trash_post( $key );
			$this->vbp->info_message( sprintf( __( 'Schedule Checker: DailyMotion video %s not found. Sending post %s to trash.', 'autovid-profit' ), $value, $key ), 'updated', 'critical' );
		}
		return 0;
	}

	/**
	 * Creates batches of DailyMotion ids from save posts to check
	 */
	public function check_for_deleted_videos() {
		$this->vbp->info_message( sprintf( __( 'DailyMotion %s Start', 'autovid-profit' ), __FUNCTION__ ), 'updated', 'utility_funcs' );
		// get comma delimited videoIDs from post metadata
		$getMore = true;
		$offset = 0;
		$chunksize = apply_filters( 'vb_posts_chunksize', 1000 );

		while ( $getMore ) {
		$posts = $this->vbp->get_posts_by_site( 'DailyMotion', $offset );
		$num_results = count( $posts );
		$num=0;
		foreach ( $posts as $post ) {
			if ( ! $num ) {
				$videoIDs = array();
			}
			$videoID = get_post_meta( $post->ID, 'VideoID', TRUE );
			if ( ! $videoID ) {	// mvb compatability check:
				$videoID = get_post_meta( $post->ID, 'mvb_vid_code', TRUE );
			}
			if ( ! $videoID ) {
				continue; 
			}
			$videoIDs[$post->ID] = $videoID;
			if ( ++$num >= $this->batch_limit ) {
				$num = $this->check_for_deleted_batch( $videoIDs );
				$this->vbp->extend_execution_time();
			}
		} // end foreach posts
			$offset += $chunksize;
			$query_args['offset'] = $offset;
			if ( $num_results < $chunksize ) $getMore = false;
			$this->vbp->extend_execution_time();
		} // end getMore
		if ( $num ) {
			$this->check_for_deleted_batch( $videoIDs );	// still some left
		}
		$this->vbp->info_message( sprintf( __( 'DailyMotion %s End', 'autovid-profit' ), __FUNCTION__ ), 'updated', 'utility_funcs' );
	}

    } // END class Autovid_Profit_DailyMotion
} // END if ( ! class_exists( 'Autovid_Profit_DailyMotion' ) )
?>
