<?php

/*
 * Prevent Direct Access
 */
defined( 'ABSPATH' ) or die( "Sorry, Can not access page directly." );

/**
 * Class for Autovid Profit Pro Hulu support
 */
if ( ! class_exists( 'Autovid_Profit_Hulu' )) {
    class Autovid_Profit_Hulu {

	private $vbp = null;			// will point to main AutoVid_Profit instance
        private $query_fields = array();        // options for current query

        /* The 'Hulu' string passed to the constructor is what Hulu calls the `dp_id`.
        It is what most would call a client id. It is required by all Hulu endpoints. */
        protected $client_id;
        
	private $APIs = array(
		'search.list'		=> 'http://m.hulu.com/search',
		'video.data'		=> 'http://m.hulu.com/videos'
		);

	private $batch_limit = 50;             // max amount we can request per query

	/**
	 * Create Hulu video source
	 * Point back to Autovid Profit Pro object to use common functions
	 * Save the query fields for easy access
	 */
	public function __construct( $vbp, $query_fields, $client_id='Hulu' ) {
		$this->vbp = $vbp;
		$this->query_fields = $query_fields;
		$this->client_id = $client_id;
	}

	/**
	 * Shortcut to proper API url
	 */
	private function getApi( $type ) {
		return $this->APIs[$type];
	}

	/**
	 * Make the query and check for errors.
	 */
	private function queryApi( $url ) {
		$this->vbp->info_message( __FUNCTION__ . ': ' . htmlentities( $url ), 'updated', 'debug' );

		$response = wp_remote_get( $url );
		if ( is_wp_error( $response ) ) {
			return $this->vbp->info_message( sprintf( __( 'WP error %s in %s(%s) - %s ', 'autovid-profit' ), $response->get_error_code(), __FUNCTION__, $url, $response->get_error_message() ) );
		}

		$data = new SimpleXMLElement( $response['body'] );

		//MARK not sure if Hulu sets this on error...
		if ( isset( $data->error ) ) {
			return $this->vbp->info_message( sprintf( __( '%s(%s) - Hulu returned %s', 'autovid-profit' ), __FUNCTION__, $url, $data->error) );
		}

		return $data;
	}

	/**
	 * Extract video details into our generic videoInfo array
	 * Note video detail fields will vary depending on if query type was search or video ID! Check for both!
	 */
	private function get_video_info( $video ) {
		$videoInfo = array();

		$videoInfo['videoID'] = isset( $video->id ) ? (string) $video->id : '';
		$videoInfo['url'] = 'http://www.hulu.com/watch/' . $videoInfo['videoID'];
		$videoInfo['orig_title'] = $videoInfo['title'] = isset( $video->title ) ? (string) $video->title : '';
		$videoInfo['orig_desc'] = $videoInfo['desc'] = isset( $video->description ) ? (string) $video->description : '';
		$videoInfo['association'] = isset( $video->{"show-name"} ) ? (string) $video->{"show-name"} : '';
		$videoInfo['tags'] = $videoInfo['association'];
		$videoInfo['img'] = $this->get_best_thumbnail( $video );

		// Make a separate call to Hulu oembed for embed code
		$url = "http://www.hulu.com/api/oembed.json?url=" . htmlentities( $videoInfo['url'] );
		$videoInfo['videoEmbed'] = $this->vbp->queryoEmbed( $url );

		$publishedAt = isset( $video->{"released-at"} ) ? (string) $video->{"released-at"} : current_time( 'Y-m-d H:i:s' );
		$publishedAt = isset( $video->released_at ) ? (string) $video->released_at : $publishedAt;

		$date = $this->vbp->getDateTime( $publishedAt, $videoInfo['url'] );
		$videoInfo['publishedAt'] = isset( $date ) ? $date->format( 'Y-m-d H:i:s' ) : $publishedAt;
		$videoInfo['publishedAt8601'] = isset( $date ) ? $date->format( 'c' ) : '';

		$d = isset( $video->duration ) ? (int) $video->duration : 0;
		$videoInfo['duration'] = '';
		if ( $d >= 3600 ) {
			$videoInfo['duration'] .= sprintf( '%02d:', ($d/3600) );
		}
		$videoInfo['duration'] .= sprintf('%02d:%02d', ($d/60%60), $d%60);

		$videoInfo['sourceCategory'] = '';
		if ( TRUE == $this->query_fields['cAddSourceCategory'] ) {

			$cat = isset( $video->show->genre ) ? (string) $video->show->genre : '';
			$cat = isset( $video->parent_channel_name ) ? (string) $video->parent_channel_name : $cat;
			$videoInfo['sourceCategoryName'] = $cat;
			$videoInfo['sourceCategory'] = $this->vbp->create_the_category( $cat );
		}

		// Hulu specific
		$videoInfo['show-name'] = isset( $video->{"show-name"} ) ? (string) $video->{"show-name"} : '';
		$videoInfo['show-name'] = isset( $video->show_name ) ? (string) $video->show_name : $videoInfo['show-name'];

		$videoInfo['videoSource'] = 'Hulu';

		$videoInfo = apply_filters( 'get_video_info', $videoInfo );

		return $videoInfo;
	}

	/**
	 * Takes an array of video details to process and create posts
	 */
	private function save_videos( $videoDetails ) {
                if ( ! $videoDetails ) {
			return $this->vbp->info_message( sprintf( __( 'Error: %s - no data to save.', 'autovid-profit' ), __FUNCTION__ ) );
                }
		foreach ( $videoDetails->video as $video ) {
			$videoInfo = $this->get_video_info( $video );
			$this->vbp->info_message( sprintf( __( 'Checking Hulu video: [%s] at %s', 'autovid-profit' ), $videoInfo['title'], $videoInfo['url'] ), 'updated', 'video_import' );

			$postID = $this->vbp->save_the_video( $this->query_fields, $videoInfo );

			if ( ! $postID ) continue;

			$this->vbp->publish_the_video( $postID, $this->query_fields, $videoInfo );
		}
	}

	private function grab_single_video() {
		$query_args = array(
			'dp_identifier'		=> $this->client_id,
			'video_id'		=> $this->query_fields['qAssoc']
			);
		$url = $this->getApi( 'video.data' ) . '?' . http_build_query( $query_args );
		$response = $this->queryApi( $url );
		if ( empty( $response->video ) ) {
			$this->vbp->info_message( __( 'No results returned for given query.', 'autovid-profit' ), 'updated', 'critical' );
		}
		else {
			$this->save_videos( $response );
		}
	}

	/**
	 * Make a batch request for videos
	 */
	private function grab_videos_batch( $url ) {
		$response = $this->queryApi( $url );
		if ( ! empty( $response->videos ) ) {
			$this->save_videos( $response->videos );
		}
		else {
			$this->vbp->info_message( __( 'No results returned for given query.', 'autovid-profit' ), 'updated', 'critical' );
		}
		return 1;
	}

	/**
	 * Process the video query and break down into batches
	 * Hulu does not have an official API
	 */
	public function grab_videos() {

		$keyphrase = $this->query_fields['qKeyphrase'];
		$assoc = $this->query_fields['qAssoc'];
		$assocType = $this->query_fields['qAssocType'];

		if ( empty( $keyphrase ) && ( empty( $assoc ) || empty( $assocType ) ) ) {
                        return $this->vbp->info_message( __( 'Error: No Hulu keyphrase or association not set properly.', 'autovid-profit' ), 'error', 'critical' );
                }

		if ( $assocType == 'video' && $assoc ) {
			return $this->grab_single_video();
		}

		$totalVids = $this->query_fields['qNumVideos'];

		$query_args = array(
			'dp_identifier'		=> $this->client_id,
			'query'			=> $keyphrase,
			'items_per_page'	=> $totalVids > $this->batch_limit ? $this->batch_limit : $totalVids,
			'page'			=> 1
		);

		$query_args = apply_filters( 'hulu_search_query', $query_args, $this->query_fields );

		while ( $totalVids > 0 ) {
			$url = $this->getApi( 'search.list' ) . '?' . http_build_query( $query_args );
			if ( ! $this->grab_videos_batch( $url ) ) {
				break;
			}
			$totalVids -= $query_args['items_per_page'];
			$query_argsp['items_per_page'] = $totalVids > $this->batch_limit ? $this->batch_limit : $totalVids;
			$query_args['page']++;
		}
	}

	/**
	 * Checks video ids to see if they're been deleted from Hulu
	 * No batch checking allowed - have to query each video individually :(
	 */
	public function check_for_deleted_videos() {
		$this->vbp->info_message( sprintf( __( 'Hulu %s Start', 'autovid-profit' ), __FUNCTION__ ), 'updated', 'utility_funcs' );

		$getMore = true;
		$offset = 0;
		$chunksize = apply_filters( 'vb_posts_chunksize', 1000 );

		while ( $getMore ) {
		$posts = $this->vbp->get_posts_by_site( 'Hulu', $offset );
		$num_results = count( $posts );
		foreach ( $posts as $post ) {
			$videoID = get_post_meta( $post->ID, 'VideoID', TRUE );
			if ( ! $videoID ) {     // mvb compatability check:
                                $videoID = get_post_meta( $post->ID, 'mvb_vid_code', TRUE );
                        }
			if ( ! $videoID ) {
				continue;
			}
			$url = get_post_meta( $post->ID, 'VideoUrl', TRUE );
			if ( ! $url ) {     // mvb compatability check:
                                $url = get_post_meta( $post->ID, 'mvb_vid_url', TRUE );
                        }
			$this->vbp->info_message( sprintf( __( 'Schedule Checker: checking Hulu video ID %s, url: %s ', 'autovid-profit' ), $videoID, $url ), 'updated', 'utility_funcs' );
			$query_args = array(
				'dp_identifier'		=> $this->client_id,
				'video_id'		=> $videoID
				);
			$url = $this->getApi( 'video.data' ) . '?' . http_build_query( $query_args );
			$response = $this->queryApi( $url );
			if ( empty( $response->video ) ) {
				$postID = apply_filters( 'vb_delete_video', $post->ID );
				if ( ! $postID ) {
					continue;
				}
				wp_trash_post( $postID );
				$this->vbp->info_message( sprintf( __( 'Schedule Checker: Hulu video %s not found. Sending post %s to trash.', 'autovid-profit' ), $videoID, $postID ), 'updated', 'critical' );
			}
		} // end foreach posts
			$offset += $chunksize;
			$query_args['offset'] = $offset;
			if ( $num_results < $chunksize ) $getMore = false;
			$this->vbp->extend_execution_time();
		} // end getMore
		$this->vbp->info_message( sprintf( __( 'Hulu %s End', 'autovid-profit' ), __FUNCTION__ ), 'updated', 'utility_funcs' );
	}

	/**
	 * Gets best thumbnail for a single video request
	 * Weird - search query API call only returns a single thumbnail_url field...
	 */
	private function get_best_thumbnail ( $video ) {
		$img = array();
		$match = false;

		if ( ! $match && isset( $video->thumbnail_url_4x3_small ) ) $img[] = $video->thumbnail_url_4x3_small;
		if ( $this->query_fields['qExtraParams2'] == '4x3_small' ) $match = true;

		if ( ! $match && isset( $video->thumbnail_url_4x3_medium ) ) $img[] = $video->thumbnail_url_4x3_medium;
		if ( $this->query_fields['qExtraParams2'] == '4x3_medium' ) $match = true;

		if ( ! $match && isset( $video->thumbnail_url_4x3_large ) ) $img[] = $video->thumbnail_url_4x3_large;
		if ( $this->query_fields['qExtraParams2'] == '4x3_large' ) $match = true;

		if ( ! $match && isset( $video->thumbnail_url_16x9_medium ) ) $img[] = $video->thumbnail_url_16x9_medium;
		if ( $this->query_fields['qExtraParams2'] == '16x9_medium' ) $match = true;

		if ( ! $match && isset( $video->thumbnail_url_16x9_large ) ) $img[] = $video->thumbnail_url_16x9_large;
		if ( $this->query_fields['qExtraParams2'] == '16x9_large' ) $match = true;

		// catchall
		if ( isset( $video->{"thumbnail-url"} ) ) $img[] = (string) $video->{"thumbnail-url"};

		$img = array_reverse( $img );

		$image = $this->vbp->verify_thumbnails( $img );

		return $image;
	}

    } // END class Autovid_Profit_Hulu
} // END if ( ! class_exists( 'Autovid_Profit_Hulu' ) )
?>
