<?php

/*
 * Prevent Direct Access
 */
defined( 'ABSPATH' ) or die( "Sorry, Can not access page directly." );

/**
 * Class for Autovid Profit Spotify support
 * https://www.spotify.com/developers/
 */
if ( ! class_exists( 'Autovid_Profit_Spotify' )) {
    class Autovid_Profit_Spotify {

	private $vbp = null;			// will point to main AutoVid_Profit instance
        private $query_fields = array();        // options for current query

	private $batch_limit = 50;             // max amount we can request per query

	/**
	 * Create Spotify video source
	 * Point back to Autovid Profit object to use common functions
	 * Save the query fields for easy access
	 */
	public function __construct( $vbp, $query_fields ) {
		$this->vbp = $vbp;
		$this->query_fields = $query_fields;
	}

        /**
         * Make the query and check for errors.
         */
        private function queryApi( $url )
        {
                $this->vbp->info_message( __FUNCTION__ . ': ' . htmlentities( $url ), 'updated', 'debug' );
		$response = wp_remote_get( $url );
                if ( is_wp_error( $response ) ) {
			return $this->vbp->info_message( sprintf( __( 'WP error %s in %s(%s) - %s ', 'autovid-profit' ), $response->get_error_code(), __FUNCTION__, $url, $response->get_error_message() ) );
                }
                $data = json_decode( wp_remote_retrieve_body( $response ) );

                if ( isset( $data->errors ) ) {
                        return $this->vbp->info_message( sprintf( __( '%s(%s) - Spotify API returned %s', 'autovid-profit' ), __FUNCTION__, $url, $data->errors[0]->error_message ) );
                }
                if ( empty( $data ) )  {
                        $this->vbp->info_message( sprintf( __( '%s(%s) - Spotify Error: no results found for query', 'autovid-profit' ), __FUNCTION__, $url ) );
                }
                return $data;
        }


        /*
         * Save the video info as a WordPress post
         */
	private function save_spotify_entry( $videoInfo ) {

		$postID = $this->vbp->save_the_video( $this->query_fields, $videoInfo );

		if ( ! $postID ) return;

		if ( ! empty( $this->query_fields['qNumComments'] ) && $videoInfo['commentCount'] ) {
			$this->grab_comments( $videoInfo['url'], $postID );
		}

		$this->vbp->publish_the_video( $postID, $this->query_fields, $videoInfo );
	}

        /**
         * Find the best match thumbnail to what the user requested. 
	 * If it is not defined this code will cascade to the closest match.
         */
	private function get_best_thumbnail( $item ) {
		$img = array();
		$match = false;

		if ( isset( $item->images[0]->url ) ) $img[] = $item->images[0]->url;
		if ( $this->query_fields['qExtraParams2'] == 'large' ) $match = true;

		if ( ! $match && isset( $item->images[1]->url ) ) $img[] = $item->images[1]->url;
		if ( $this->query_fields['qExtraParams2'] == 'medium' ) $match = true;

		if ( ! $match && isset( $item->images[2]->url ) ) $img[] = $item->images[2]->url;

		$img = array_reverse( $img );

		$image = $this->vbp->verify_thumbnails( $img );

		return $image;
	}

        /**
         * Extract album,artist,playlist, or track details into our generic videoInfo array
         */
        private function get_video_info( $item ) {
		$assocType = $this->query_fields['qAssocType'];
		$videoInfo = array();

		if ( $assocType == 'album' ) {
			$videoInfo['img'] = $this->get_best_thumbnail( $item );
		}
		else if ( $assocType == 'artist' ) {
			$videoInfo['img'] = $this->get_best_thumbnail( $item );
		}
		else if ( $assocType == 'playlist' ) {
			$videoInfo['association'] = isset( $item->owner->id ) ? $item->owner->id : '';
			$videoInfo['img'] = $this->get_best_thumbnail( $item );
		}
		else if ( $assocType == 'track' ) {
			$videoInfo['association'] = isset( $item->album->name ) ? $item->album->name : '';
			$videoInfo['img'] = $this->get_best_thumbnail( $item->album );
		}

		$videoInfo['orig_title'] = $videoInfo['title'] = isset( $item->name ) ? $item->name : '';
		$videoInfo['orig_desc'] = $videoInfo['desc'] = '';
		$videoInfo['videoID'] = isset( $item->uri ) ? $item->uri : '';
		$videoInfo['url'] = isset( $item->external_urls->spotify ) ? $item->external_urls->spotify : '';

		$this->vbp->info_message( sprintf( __( 'Checking Spotify item: [%s] at %s', 'autovid-profit' ), $videoInfo['title'], $videoInfo['url'] ), 'updated', 'video_import' );

		$videoInfo['duration'] = isset( $item->duration_ms ) ? $item->duration_ms : 0; 
		$videoInfo['duration'] = $this->vbp->seconds_to_time( $videoInfo['duration'] / 1000 );
		$videoInfo['viewCount'] = 0;
		$videoInfo['likeCount'] = 0;
		$videoInfo['dislikeCount'] = 0;
		$videoInfo['commentCount'] = 0;

	

		// Make a separate call to Spotify oembed for embed code 
		$url = "https://embed.spotify.com/oembed?url=" . htmlentities( $videoInfo['url'] );
		$videoInfo['videoEmbed'] = $this->vbp->queryoEmbed( $url );

		// spotify does not have a published time, use now
		$date = $this->vbp->getDateTime( null, $videoInfo['url'] );
		$videoInfo['publishedAt'] = $date->format( 'Y-m-d H:i:s' );
		$videoInfo['publishedAt8601'] = isset( $date ) ? $date->format( 'c' ) : '';

		$videoInfo['tags'] = '';
		if ( TRUE == $this->query_fields['pGrabTags'] && isset( $item->artists ) ) {
			$tags = array();
			if ( isset( $item->album->name ) ) {
				$tags[] = $item->album->name;
			}
			foreach ( $item->artists as $tag ) {
				$tags[] = $tag->name;
			}
			$videoInfo['tags'] = implode( ",", $tags );
		}

		$videoInfo['videoSource'] = 'Spotify';

		$videoInfo = apply_filters( 'get_video_info', $videoInfo );

		$this->vbp->info_message( sprintf( __( '%s: %s', 'autovid-profit' ), __FUNCTION__, htmlentities( print_r($videoInfo,true) ) ), 'updated', 'debug' );

		return $videoInfo;
	}


	public function grab_videos_batch( $url ) {

		$result = $this->queryApi( $url );
		if ( empty( $result ) ) return null;

		$assocType = $this->query_fields['qAssocType'];

		if ( $assocType == 'album' ) {
			$items = isset( $result->albums->items ) ? $result->albums->items : array();
		}
		else if ( $assocType == 'artist' ) {
			$items = isset( $result->artists->items ) ? $result->artists->items : array();
		}
		else if ( $assocType == 'playlist' ) {
			$items = isset( $result->playlists->items ) ? $result->playlists->items : array();
		}
		else if ( $assocType == 'track' ) {
			$items = isset( $result->tracks->items ) ? $result->tracks->items : array();
		}

		foreach ( $items as $item ) {
			$videoInfo = $this->get_video_info( $item );
			$this->save_spotify_entry( $videoInfo );
		}
	}

	public function grab_videos() {
		$totalVids = $this->query_fields['qNumVideos'];
		$keyphrase = urlencode( $this->query_fields['qKeyphrase'] );
		$assocType = $this->query_fields['qAssocType'];

		if ( empty( $keyphrase ) ) {
			return $this->vbp->info_message( __( 'Error: No Spotify keyphrase or association not set properly.', 'autovid-profit' ), 'error', 'critical' );
		}

		// Spotify wants raw url encoding
		$keyphrase = urldecode( $keyphrase );
		$base_url = "http://api.spotify.com/v1/search/?&q=" . rawurlencode( $keyphrase );
		$base_url .= "&type={$assocType}";

		$base_url = apply_filters( 'spotify_search_query', $base_url, $this->query_fields );

		$offset = 0;
		while ( $totalVids > $offset ) {
			$numLeft = $totalVids - $offset;
			$limit = $numLeft > $this->batch_limit ? $this->batch_limit : $numLeft;
			$url = $base_url . "&limit={$limit}&offset={$offset}";
			$this->grab_videos_batch( $url );
			$offset += $limit;
		}
	}

	/**
	 * Spotify 
	 */

	private function check_for_deleted_batch( $spotIDs, $type ) {
		if ( empty( $spotIDs ) ) return 0;  // shouldn't happen

		$chunks = array_chunk( $spotIDs, $this->batch_limit );
		foreach ( $chunks as $chunk ) {

			foreach ( $chunk as $key => $value ) {
				$list[] = $value;
			}
			$commaList = implode( ',', $list );
			$this->vbp->info_message( sprintf( __( 'Schedule Checker: checking Spotify %s IDs: %s ', 'autovid-profit' ), $type, $commaList ), 'updated', 'utility_funcs' );

			if ( $type == 'album' ) {
				$url = "https://api.spotify.com/v1/albums/?ids={$commaList}";
				$result = $this->queryApi( $url );
				$items = isset( $result->albums ) ? $result->albums : array();
			}
			else if ( $type == 'artist' ) {
				$url = "https://api.spotify.com/v1/artists/?ids={$commaList}";
				$result = $this->queryApi( $url );
				$items = isset( $result->artists ) ? $result->artists : array();
			}
			else if ( $type == 'track' ) {
				$url = "https://api.spotify.com/v1/tracks/?ids={$commaList}";
				$result = $this->queryApi( $url );
				$items = isset( $result->tracks ) ? $result->tracks : array();
			}
			// returned items are GOOD. remove them from the list.
			foreach ( $items as $item ) {
				if ( ( $key = array_search( $item->id, $spotIDs ) ) !== FALSE ) {
					unset( $spotIDs[$key] );
				}
			}

			// any remaining IDs means they weren't found by Spotify
			foreach ( $spotIDs as $key => $value ) {
				$key = apply_filters( 'vb_delete_video', $key );
				if ( ! $key ) continue;
				wp_trash_post( $key );
				$this->vbp->info_message( sprintf( __( 'Schedule Checker: Spotify %s ID %s not found. Sending post %s to trash.', 'autovid-profit' ), $type, $value, $key ), 'updated', 'critical' );
			}
		}
		$this->vbp->extend_execution_time();
	}


	public function check_for_deleted_videos() {
		$this->vbp->info_message( sprintf( __( 'Spotify %s Start', 'autovid-profit' ), __FUNCTION__ ), 'updated', 'utility_funcs' );

		$getMore = true;
		$offset = 0;
		$chunksize = apply_filters( 'vb_posts_chunksize', 1000 );

		while ( $getMore ) {
		$posts = $this->vbp->get_posts_by_site( 'Spotify', $offset );
		$num_results = count( $posts );
		$albumIDs = $artistIDs = $trackIDs = array();
		foreach ( $posts as $post ) {
			$spotifyID = get_post_meta( $post->ID, 'VideoID', TRUE );
			if ( ! $spotifyID ) continue;
			$fields = explode( ':', $spotifyID );	// spotify:[TYPE]:[ID]
			if ( $fields[0] != 'spotify' ) continue;	// should never happen

			if ( $fields[1] == 'album' ) 	$albumIDs[$post->ID] 	= $fields[2];
			if ( $fields[1] == 'artist' ) 	$artistIDs[$post->ID] 	= $fields[2];
			if ( $fields[3] == 'playlist' ) {
				// Spotify requires OAuth for some reason. @#$% that
			}
			if ( $fields[1] == 'track' ) 	$trackIDs[$post->ID] 	= $fields[2];

		} // end foreach posts
			$offset += $chunksize;
			$query_args['offset'] = $offset;
			if ( $num_results < $chunksize ) $getMore = false;
			$this->vbp->extend_execution_time();

		if ( ! empty( $albumIDs ) ) {
			$this->check_for_deleted_batch( $albumIDs, 'album' );
		}
		if ( ! empty( $artistIDs ) ) {
			$this->check_for_deleted_batch( $artistIDs, 'artist' );
		}
		if ( ! empty( $trackIDs ) ) {
			$this->check_for_deleted_batch( $trackIDs, 'track' );
		}
		} // end getMore
		$this->vbp->info_message( sprintf( __( 'Spotify %s End', 'autovid-profit' ), __FUNCTION__ ), 'updated', 'utility_funcs' );
	}

    } // END class Autovid_Profit_Spotify
} // END if ( ! class_exists( 'Autovid_Profit_Spotify' ) )


?>
