<?php

/*
 * Prevent Direct Access
 */
defined( 'ABSPATH' ) or die( "Sorry, Can not access page directly." );

/**
 * Class for Autovid Profit Pro Vimeo support
 */
if ( ! class_exists( 'Autovid_Profit_Vimeo' )) {
    class Autovid_Profit_Vimeo {

	private $vbp = null;			// will point to main AutoVid_Profit instance
        private $query_fields = array();        // options for current query
	private $access_token = null;		// Vimeo unauthorized access token
	private $APIs = array(
		'video.data' 		=> 'https://api.vimeo.com/videos/',
		'videos.list' 		=> 'https://api.vimeo.com/videos',
		'categories.list' 	=> 'https://api.vimeo.com/categories/',
		'channels.list' 	=> 'https://api.vimeo.com/channels/',
		'groups.list' 		=> 'https://api.vimeo.com/groups/',
		'token.authorize' 	=> 'https://api.vimeo.com/oauth/authorize/client',
        );

	private $batch_limit = 50;             // max amount we can request per query

	/**
	 * Create Vimeo video source
	 * Point back to Autovid Profit Pro object to use common functions
	 * Save the query fields for easy access
	 * Requires an app access token
	 */
	public function __construct( $vbp, $query_fields, $access_token ) {
		$this->vbp = $vbp;
		$this->query_fields = $query_fields;
		$this->access_token = $access_token;
	}

	/**
	 * Shortcut to proper API url
	 */
	private function getApi( $type ) {
		return $this->APIs[$type];
	}

	/**
	 * Make the query and check for errors.
	 */
	private function queryApi( $url ) {
		$this->vbp->info_message( __FUNCTION__ . ': ' . htmlentities( $url ), 'updated', 'debug' );

		$headers = array (
			'Accept' 	=> 'application/vnd.vimeo.*+json; version=3.2',
			'User-Agent' 	=> 'video blogster pro 1.0; (http://developer.vimeo.com/api/docs)',
			'Authorization'	=> 'Bearer ' . $this->access_token
			);

		$response = wp_remote_get( $url, array( 'headers' => $headers ) );
		if ( is_wp_error( $response ) ) {
			return $this->vbp->info_message( sprintf( __( 'WP error %s in %s(%s) - %s ', 'autovid-profit' ), $response->get_error_code(), __FUNCTION__, $url, $response->get_error_message() ) );
		}
		$data = json_decode( wp_remote_retrieve_body( $response ) );
		if ( isset( $data->error ) ) {
			$this->vbp->info_message( sprintf( __( '%s(%s) - Vimeo API returned %s', 'autovid-profit' ), __FUNCTION__, $url, $data->error ) );
		}
		return $data;
	}

	private function get_best_thumbnail( $video ) {
		$img = array();

		foreach ( $video->pictures->sizes as $pic ) {
			$img[] = $pic->link;
			$width = $pic->width . "px";
			if ( $this->query_fields['qExtraParams2'] == $width ) {
				break;
			}
		}

		$img = array_reverse( $img );

		$image = $this->vbp->verify_thumbnails( $img );

		return $image;
	}

	/**
	 * Extract video details into our generic videoInfo array
	 */
	private function get_video_info( $video ) {
		$videoInfo = array();

		$videoInfo['orig_title'] = $videoInfo['title'] = isset( $video->name ) ? $video->name : '';
		$videoInfo['orig_desc'] = $videoInfo['desc'] = isset( $video->description ) ? $video->description : '';
		$videoInfo['association'] = isset( $video->user->name ) ? $video->user->name : '';
		$videoInfo['img'] = $this->get_best_thumbnail( $video );
		$videoInfo['url'] = isset( $video->link ) ? $video->link : '';

                // Make a separate call to Vimeo oembed for embed code
                $url = "http://vimeo.com/api/oembed.json?url=" . htmlentities( $videoInfo['url'] );
		$videoInfo['videoEmbed'] = $this->vbp->queryoEmbed( $url );

		$uri = isset( $video->uri ) ? $video->uri : 0;
		if ( preg_match( "/\s*(\d+)$/", $uri, $matches ) ) {
			$videoInfo['videoID'] = $matches[1];
		}

		$publishedAt = isset( $video->created_time ) ?  $video->created_time : current_time( 'Y-m-d H:i:s' );
		$date = $this->vbp->getDateTime( $publishedAt, $videoInfo['url'] );
                $videoInfo['publishedAt'] = isset( $date ) ? $date->format( 'Y-m-d H:i:s' ) : $publishedAt;
		$videoInfo['publishedAt8601'] = isset( $date ) ? $date->format( 'c' ) : '';

		$d = isset( $video->duration ) ? $video->duration : 0;
		$videoInfo['duration'] = '';
		if ( $d >= 3600 ) {
			$videoInfo['duration'] .= sprintf( '%02d:', ($d/3600) );
		}
		$videoInfo['duration'] .= sprintf('%02d:%02d', ($d/60%60), $d%60);

		if ( TRUE == $this->query_fields['pGrabTags'] ) {
			foreach ( $video->tags as $tag ) {
				$tags[] = $tag->name;
			}
		}
		$videoInfo['tags'] = ! empty( $tags ) ? implode( ',', $tags ) : '';

		$videoInfo['channel'] = isset( $video->user->name ) ? $video->user->name : '';

		$videoInfo['sourceCategory'] = '';
		// vimeo videos don't have this info, just use the form query
		if ( TRUE == $this->query_fields['cAddSourceCategory'] && ! empty ( $this->query_fields['qAssoc'] ) ) {
			$videoInfo['sourceCategoryName'] = $this->query_fields['qAssoc'];
			$videoInfo['sourceCategory'] = $this->vbp->create_the_category( $this->query_fields['qAssoc'] );
		}

		$videoInfo['viewCount'] = isset( $video->stats->plays ) ? $video->stats->plays : 0;
		$videoInfo['likeCount'] = isset( $video->metadata->likes->total ) ? $video->metadata->likes->total : 0;

		$videoInfo['comments'] = isset( $video->metadata->connections->comments->total ) ? $video->metadata->connections->comments->total : 0;


		$videoInfo['videoSource'] = 'Vimeo';

		$videoInfo = apply_filters( 'get_video_info', $videoInfo );

		return $videoInfo;
	}

	/**
	 * Do a batch query search for up to $this->batch_limit comments at a time (Vimeo limit)
	 */
	private function grab_comments_batch( $url, $postID ) {
		$this->vbp->info_message(__FUNCTION__ . ': ' . htmlentities( $url ), 'updated', 'debug');
		$response = $this->queryApi( $url );
		$total = 0;
		foreach ( $response->data as $comment ) {
			$author = isset( $comment->user->name ) ? $comment->user->name : '';
			$content = isset( $comment->text ) ? $comment->text : '';
			if ( TRUE == $this->query_fields['cUsePublishedDate'] ) {
				$commentAt = isset( $comment->created_on ) ? $comment->created_on : current_time( 'Y-m-d H:i:s' );
				$comment_date = $this->vbp->getDateTime( $commentAt, $videoInfo['url'] );
                		$comment_date = isset( $comment_date ) ? $comment_date->format( 'Y-m-d H:i:s' ) : $commentAt;
			}
			if ( TRUE == $this->query_fields['pRemoveCommUrls'] ) {
				$content = $this->vbp->remove_urls( $content );
			}
			if ( TRUE == $this->query_fields['pLinkifyCommUrls'] ) {
				$content = $this->vbp->linkify_filtered( $content );
			}


			$commentdata = array(
				'comment_post_ID'       => $postID,
				'comment_author'        => $author,
				'comment_author_email'  => '',
				'comment_author_url'  	=> '',
				'comment_content'       => $content,
				'comment_type'  	=> '',
				'comment_date'          => ( TRUE == $this->query_fields['cUsePublishedDate'] ) ? $comment_date : current_time( 'Y-m-d H:i:s' ),
				'comment_approved'      => 1
			);

			$commentdata = apply_filters( 'vimeo_comment', $commentdata );
			if ( empty( $commentdata ) ) {
				continue;
			}
			$comment_id = wp_insert_comment( $commentdata );
			if ( $comment_id ) {
				$total++;
			}
		}
		$this->vbp->info_message( sprintf( __( '%s comments imported successfully to post id %s.', 'autovid-profit' ), $total, $postID ), 'updated', 'video_import' );

	return isset( $data->paging->next ) ? $data->paging->next : 0;
	}

	/**
	 * Grab and save comments for a video
	 * Toggle comment approval and email notification if set
	 * Break up process into batches
	 */
	private function grab_comments( $videoID, $postID ) {
		
       		$cm = get_option( 'comment_moderation' );
       		$cn = get_option( 'comment_notification' );
		// temporarily turn off manual approval and email notification
       		if ( 1 == $cm ) {
			update_option( 'comment_moderation', FALSE );
		}
       		if ( 1 == $cn ) {
			update_option( 'comment_notification', FALSE );
		}

		$totalComments = $this->query_fields['qNumComments'];
        	$query_args = array(
                	'per_page'	=> $totalComments > $this->batch_limit || $totalComments < 0 ? $this->batch_limit : $totalComments,
                	'direction'	=> 'desc',
			'page'		=> 1
        	);

		$query_args = apply_filters( 'vimeo_comments_query', $query_args, $this->query_fields );

		$next = $this->getApi( 'video.data' ) . $videoID . '/comments?' . http_build_query( $query_args );
		while ( $totalComments ) {
			$next = $this->grab_comments_batch( $next, $postID );
			if ( ! $next ) {
				break;	// no more comments
			}
			$totalComments -= $query_args['per_page'];
                	$query_args['per_page'] = $totalComments > $this->batch_limit || $totalComments < 0 ? $this->batch_limit : $totalComments;
		}

		// turn back on manual approval and email notifications, if they were set set
       		if ( 1 == $cm ) {
			update_option( 'comment_moderation', $cm );
		}
       		if ( 1 == $cn ) {
			update_option( 'comment_notification', $cn );
		}
	}

	/**
	 * Takes an array of video details to process and create posts
	 */
	private function save_videos( $videoDetails ) {
                if ( ! $videoDetails ) {
			return $this->vbp->info_message( sprintf( __( 'Error: %s - no data to save.', 'autovid-profit' ), __FUNCTION__ ) );
                }
		foreach ( $videoDetails as $video ) {
			$videoInfo = $this->get_video_info( $video );

			$this->vbp->info_message( sprintf( __( 'Checking Vimeo video: [%s] at %s', 'autovid-profit' ), $videoInfo['title'], $videoInfo['url'] ), 'updated', 'video_import' );

			$postID = $this->vbp->save_the_video( $this->query_fields, $videoInfo );

			if ( ! $postID ) continue;

			if ( ! empty( $this->query_fields['qNumComments'] ) && $videoInfo['comments'] ) {
				$this->grab_comments( $videoInfo['videoID'], $postID );
			}

			$this->vbp->publish_the_video( $postID, $this->query_fields, $videoInfo );
		}
	}

	/**
	 * Grab a single video by the video ID stored in qAssoc.
	 */
	private function grab_single_video() {
		$url = $this->getApi( 'video.data' ) . $this->query_fields['qAssoc'];
		$response = $this->queryApi( $url );
		if ( ! isset( $response->error ) ) { 
			$this->save_videos( array( $response ) );
		}
	}

	/**
	 * Make a batch request for videos
	 */
	private function grab_videos_batch( $url ) {
		$response = $this->queryApi( $url );
		if ( ! empty( $response->data ) ) {
			$this->save_videos( $response->data );
		}
		else {
			$this->vbp->info_message( __( 'No results returned for given query.', 'autovid-profit' ), 'updated', 'critical' );
		}
		return 1;
	}

	/**
	 * Process the video query and break down into batches
	 * https://developer.vimeo.com/api/endpoints/videos#
	 */
	public function grab_videos() {

		$totalVids = $this->query_fields['qNumVideos'];

		$keyphrase = $this->query_fields['qKeyphrase'];
		$assoc = $this->query_fields['qAssoc'];
		$assocType = $this->query_fields['qAssocType'];

		if ( empty( $keyphrase ) && ( empty( $assoc ) || empty( $assocType ) ) ) {
                        return $this->vbp->info_message( __( 'Error: No Vimeo keyphrase or association not set properly.', 'autovid-profit' ), 'error', 'critical' );
                }

		if ( $assoc && $assocType == 'category' ) {
			$base_url = $this->getApi( 'categories.list' ) . $assoc . '/videos';
		}
		else if ( $assoc && $assocType == 'channel' ) {
			$base_url = $this->getApi( 'channels.list' ) . $assoc . '/videos';
		}
		else if ( $assoc && $assocType == 'group' ) {
			$base_url = $this->getApi( 'groups.list' ) . $assoc . '/videos';
		}
		else if ( $assoc && $assocType == 'video' ) {
			return $this->grab_single_video();
		}
		else {
			$base_url = $this->getApi( 'videos.list' );
		}

		$query_args = array(
			'per_page'		=> $totalVids > $this->batch_limit ? $this->batch_limit : $totalVids,
			'page'			=> 1,
			'sort'			=> $this->query_fields['qOrderBy'],
			'direction'		=> $this->query_fields['qOrderDirection']
		);

		if ( $assocType == 'category' || $assocType == 'channel' || $assocType == 'group' ) {
			$query_args['filter'] = 'embeddable';
			$query_args['filter_embeddable'] = 'true';
		}
		if ( $keyphrase ) {
			$query_args['query'] = $keyphrase;
		}

		$query_args = apply_filters( 'vimeo_search_query', $query_args, $this->query_fields  );

		while ( $totalVids > 0 ) {
			$url = $base_url . '?' . http_build_query( $query_args );
			if ( ! $this->grab_videos_batch( $url ) ) {
				break;
			}
			$totalVids -= $query_args['per_page'];
			$query_args['per_page'] = $totalVids > $this->batch_limit ? $this->batch_limit : $totalVids;
			$query_args['page']++;
		}
	}

	/**
	 * Checks video ids to see if they're been deleted from Vimeo
	 * No batch checking allowed - have to query each video individually :(
	 */
	public function check_for_deleted_videos() {
		$this->vbp->info_message( sprintf( __( 'Vimeo %s Start', 'autovid-profit' ), __FUNCTION__ ), 'updated', 'utility_funcs' );
		// get comma delimited videoIDs from post metadata
		$getMore = true;
		$offset = 0;
		$chunksize = apply_filters( 'vb_posts_chunksize', 1000 );

		while ( $getMore ) {
		$posts = $this->vbp->get_posts_by_site( 'Vimeo', $offset );
		$num_results = count( $posts );
		foreach ( $posts as $post ) {
			$videoID = get_post_meta( $post->ID, 'VideoID', TRUE );
			if ( ! $videoID ) {
				continue;
			}
			$url = get_post_meta( $post->ID, 'VideoUrl', TRUE );
			$this->vbp->info_message( sprintf( __( 'Schedule Checker: checking Vimeo video ID %s, url: %s', 'autovid-profit' ), $videoID, $url ), 'updated', 'utility_funcs' );
			$url = $this->getApi( 'video.data' ) . $videoID;
			$data = $this->queryApi( $url );
			if ( isset( $data->error ) && $data->error == 'The requested video could not be found' ) {
				$postID = apply_filters( 'vb_delete_video', $post->ID );
				if ( ! $postID ) {
					continue;
				}
				wp_trash_post( $postID );
				$this->vbp->info_message( sprintf( __( 'Schedule Checker: Vimeo video %s not found. Sending post %s to trash.', 'autovid-profit' ), $value, $postID ), 'updated', 'critical' );
			}
		} // end foreach posts
			$offset += $chunksize;
			$query_args['offset'] = $offset;
			if ( $num_results < $chunksize ) $getMore = false;
			$this->vbp->extend_execution_time();
		} // end getMore
		$this->vbp->info_message( sprintf( __( 'Vimeo %s End', 'autovid-profit' ), __FUNCTION__ ), 'updated', 'utility_funcs' );
	}

    } // END class Autovid_Profit_Vimeo
} // END if ( ! class_exists( 'Autovid_Profit_Vimeo' ) )
?>
