<?php

/*
 * Prevent Direct Access
 */
defined( 'ABSPATH' ) or die( "Sorry, Can not access page directly." );

/**
 * Class for Autovid Profit Pro YouTube support
 */
if ( ! class_exists( 'Autovid_Profit_YouTube' )) {
    class Autovid_Profit_YouTube {

	private $vbp=0;	// will point to main AutoVid_Profit instance
	private $apiKey=0;
	private $youtube_categories=0;		// fetch once per instance only when needed
	private $query_fields = array();	// options for current query
    	private $APIs = array(
		'videos.list' 		=> 'https://www.googleapis.com/youtube/v3/videos',
		'search.list' 		=> 'https://www.googleapis.com/youtube/v3/search',
		'channels.list' 	=> 'https://www.googleapis.com/youtube/v3/channels',
		'playlists.list' 	=> 'https://www.googleapis.com/youtube/v3/playlists',
		'playlistItems.list' 	=> 'https://www.googleapis.com/youtube/v3/playlistItems',
		'categories.list' 	=> 'https://www.googleapis.com/youtube/v3/videoCategories',
		'commentThreads.list' 	=> 'https://www.googleapis.com/youtube/v3/commentThreads'
	);
	private $batch_limit = 50;             // max amount of videos we can request per API query
	private $comments_limit = 100;         // max amount of comments we can request per API query

	/**
	 * Create YouTube video source
	 * Point back to Autovid Profit Pro object to use common functions
	 * Save the query fields for easy access
	 * Requires app API key
	 */
        public function __construct( $vbp, $query_fields, $key ) {
		$this->vbp = $vbp;
		$this->query_fields = $query_fields;
		$this->apiKey = $key;
	}

	/**
	 * Shortcut to proper API url
	 */
	private function getApi( $type )
	{
		return $this->APIs[$type];
	}

	/**
	 * Make the query and check for errors.
	 */
	private function queryApi( $url )
	{
                $this->vbp->info_message( __FUNCTION__ . ': ' . htmlentities( $url ), 'updated', 'debug' );
                $response = wp_remote_get( $url );
    		if ( is_wp_error( $response ) ) {
			return $this->vbp->info_message( sprintf( __( 'WP error %s in %s(%s) - %s ', 'autovid-profit' ), $response->get_error_code(), __FUNCTION__, $url, $response->get_error_message() ) );
		}
                $data = json_decode( wp_remote_retrieve_body( $response ) );
		if ( isset( $data->error ) ) {
			return $this->vbp->info_message( sprintf( __( '%s (%s) - YouTube API returned %s', 'autovid-profit' ), __FUNCTION__, htmlentities( $url ), $data->error->message ) );
		}
                if ( empty( $data->items ) )  {
                        $this->vbp->info_message( sprintf( __( '%s(%s) - YouTube Error: no results found for query.', 'autovid-profit' ), __FUNCTION__, htmlentities( $url ) ) );
                        $this->vbp->info_message( sprintf( __( 'Response was %s', 'autovid-profit' ), print_r($response,true) ), 'error', 'debug' );
                }
		return $data;
	}

	/**
	 * Query YouTube for existing categories
	 */
	public function grab_categories( $region = 'US' ) {
		if ( $this->youtube_categories )	{
			return $this->youtube_categories;
		}
		// if campaign is using a regionCode, use that to determine available categories instead:
		$region = ! empty( $this->query_fields['qRegionCode'] ) ? $this->query_fields['qRegionCode'] : $region;
        	$query_args = array(
                	'part'		=> 'snippet',
                	'regionCode'	=> $region,
                	'key'		=> $this->apiKey
        	);
		$url = $this->getApi( 'categories.list' ) . '?' . http_build_query( $query_args );
		$data = $this->queryApi($url);
		$this->youtube_categories = isset( $data->items ) ? $data->items : array();
		$this->youtube_categories = $this->vbp->transientChecker( 'vb_youtube_categories', $this->youtube_categories );
		return $this->youtube_categories;
	}

	/**
	 * Do a batch query request for up to $this->comments_limit comments at a time (YouTube limit)
	 */
	private function grab_comments_batch( $url, $postID ) {
		$this->vbp->info_message(__FUNCTION__ . ': ' . htmlentities( $url ), 'updated', 'debug' );
		$data = $this->queryApi( $url );
		if ( ! $data ) { // error occurred - probably timeout. Try again another time
			return $this->vbp->info_message( __( 'Query Error: Skipping this batch.', 'autovid-profit' ) );
		}
		$items = isset( $data->items ) ? $data->items : array();

		$total = 0;
		foreach ( $items as $item )  {
			$topLevelComment = $item->snippet->topLevelComment;
			$author = isset( $topLevelComment->snippet->authorDisplayName ) ? trim( $topLevelComment->snippet->authorDisplayName ) : '';
			$content = isset( $topLevelComment->snippet->textDisplay ) ? trim( $topLevelComment->snippet->textDisplay ) : '';
			$parentId = isset( $topLevelComment->id ) ? trim( $topLevelComment->id ) : '';
			if ( empty( $author ) || empty( $content ) ) {
				continue;
			}
			$comment_date = current_time( 'Y-m-d H:i:s' );
			if ( TRUE == $this->query_fields['cUsePublishedDate'] ) {
				$commentAt = isset( $topLevelComment->snippet->publishedAt ) ? $topLevelComment->snippet->publishedAt : current_time( 'Y-m-d H:i:s' );
				$commentDate = $this->vbp->getDateTime( $commentAt, $url );
				$comment_date = isset( $commentDate ) ? $commentDate->format('Y-m-d H:i:s') : $commentAt;
			}
			if ( TRUE == $this->query_fields['pRemoveCommUrls'] ) {
				$content = $this->vbp->remove_urls( $content );
			}
			if ( TRUE == $this->query_fields['pLinkifyCommUrls'] ) {
				$content = $this->vbp->linkify_filtered( $content );
			}
			$commentdata = array(
				'comment_post_ID'       => $postID,
				'comment_author'        => $author,
				'comment_author_email'  => '',
				'comment_author_url'    => '',
				'comment_content'       => $content,
				'comment_type'          => '',
				'comment_date'          => $comment_date,
				'comment_approved'      => 1
			);
			$comment_parent = $comment_id = wp_insert_comment( $commentdata );
			if ( $comment_id ) {
				$total++;
			}
			else if (! $comment_id || ! $parentId ) {
				continue;
			}
			// any replies to this top level comment?
			$replies = isset( $item->replies->comments ) ? $item->replies->comments : array();
			foreach ( $replies as $reply )  {
				$author = isset( $reply->snippet->authorDisplayName ) ? trim( $reply->snippet->authorDisplayName ) : '';
				$content = isset( $reply->snippet->textDisplay ) ? trim( $reply->snippet->textDisplay ) : '';
				if ( empty( $author ) || empty( $content ) ) {
					continue;
				}
				$comment_date = current_time( 'Y-m-d H:i:s' );
				if ( TRUE == $this->query_fields['cUsePublishedDate'] ) {
					$commentAt = isset( $topLevelComment->snippet->publishedAt ) ? $topLevelComment->snippet->publishedAt : current_time( 'Y-m-d H:i:s' );
					$comment_date = $this->vbp->getDateTime( $publishedAt, $videoInfo['url'] );
					$comment_date = isset( $comment_date ) ? $comment_date->format('Y-m-d H:i:s') : $commentAt;
				}
				if ( TRUE == $this->query_fields['pRemoveCommUrls'] ) {
					$content = $this->vbp->remove_urls( $content );
				}
				if ( TRUE == $this->query_fields['pLinkifyCommUrls'] ) {
					$content = $this->vbp->linkify_filtered( $content );
				}
				$commentdata = array(
					'comment_post_ID'       => $postID,
					'comment_author'        => $author,
					'comment_author_email'  => '',
					'comment_author_url'    => '',
					'comment_content'       => $content,
					'comment_type'          => '',
					'comment_parent'	=> $comment_parent,
					'comment_date'          => $comment_date,
					'comment_approved'      => 1
				);
				$commentdata = apply_filters( 'youtube_comment', $commentdata);
				if ( empty( $commentdata ) ) {
					continue;
				}
				$comment_id = wp_insert_comment( $commentdata );
				if ( $comment_id ) {
					$total++;
				}
			}
		}
		$this->vbp->info_message( sprintf( __( '%s total comments imported successfully to post id %s.', 'autovid-profit' ), $total, $postID ), 'updated', 'video_import' );

		return isset( $data->nextPageToken ) ? $data->nextPageToken : '';
	}

	/**
	 * Grab and save comments for a video
	 */
	private function grab_comments( $videoID, $postID ) {
		$this->vbp->info_message( sprintf( __( 'Grabbing comments for video ID %s, post ID %s', 'autovid-profit' ), $videoID, $postID ), 'updated', 'debug' );
       		$cm = get_option( 'comment_moderation' );
       		$cn = get_option( 'comment_notification' );
		// temporarily turn off manual approval and email notification
       		if ( 1 == $cm ) {
			update_option( 'comment_moderation', FALSE );
		}
       		if ( 1 == $cn ) {
			update_option( 'comment_notification', FALSE );
		}

		$totalComments = $this->query_fields['qNumComments'];
		$part = 'snippet';
		if ( TRUE === $this->query_fields['qCommentReplies'] ) {
			$part .= ', replies';
		}
        	$query_args = array(
                	'part'		=> $part,
                	'maxResults'	=> $totalComments > $this->comments_limit || $totalComments < 0 ? $this->comments_limit : $totalComments,
                	'videoId'	=> $videoID,
                	'order'		=> $this->query_fields['qCommentOrderBy'],
                	'textFormat'	=> 'plainText',
                	'key'		=> $this->apiKey
        	);

		if ( ! empty( $this->query_fields['qCommentSearchTerms'] ) ) {
			$query_args['searchTerms'] = $this->query_fields['qCommentSearchTerms'];
		}

		$query_args = apply_filters( 'youtube_comments_query', $query_args, $this->query_fields );

		while ( $totalComments ) {
			$request = $this->getApi( 'commentThreads.list' ) . '?' . http_build_query( $query_args );
			$next = $this->grab_comments_batch( $request, $postID );
			if ( ! $next ) {
				break;	// no more comments
			}
			$query_args['pageToken'] = $next;
			$totalComments -= $query_args['maxResults'];
                	$query_args['maxResults'] = $totalComments > $this->comments_limit || $totalComments < 0 ? $this->comments_limit : $totalComments;
		}

		// turn back on manual approval and email notifications, if they were set set
       		if ( 1 == $cm ) {
			update_option( 'comment_moderation', $cm );
		}
       		if ( 1 == $cn ) {
			update_option( 'comment_notification', $cn );
		}
	}


	/**
	 * Checks a batch of video ids to see if they're been deleted from YouTube
	 * called in batches of $this->batch_limit (YouTube limit). Query result list without id means that video has been deleted.
	 * Query result list without id means that video has been deleted.
	 */
	private function check_for_deleted_batch( $videoIDs ) {
		if ( empty( $videoIDs ) ) {
			return 0;
		}

		foreach ( $videoIDs as $key => $value ) {
			$list[] = $value;
		}
		$commaList = implode( ',', $list );

		$this->vbp->info_message( sprintf( __( 'Schedule Checker: checking YouTube video IDs: %s ', 'autovid-profit' ), $commaList ), 'updated', 'utility_funcs' );

        	$query_args = array(
                	'part'		=> 'id,status',
                	'id'		=> $commaList,
                	'key'		=> $this->apiKey
        	);
		$url = $this->getApi( 'videos.list' ) . '?' . http_build_query( $query_args );
		$data = $this->queryApi( $url );

		if ( ! $data ) { // error occurred - probably timeout. Try again another time
			return $this->vbp->info_message( __( 'Query Error: Skipping this batch.', 'autovid-profit' ) );
		}
		$videoDetails = isset( $data->items ) ? $data->items : array();

		// remove each ID found from query from our original list of IDs to check...
		foreach ( $videoDetails as $video )  {

			// check if video is still valid on YouTube
			// if so, remove from the GOOD list, leaving only ID's of videos that WILL be trashed
			if ( ( $key = array_search( $video->id, $videoIDs ) ) !== FALSE && ( $video->status->uploadStatus == 'processed' || $video->status->uploadStatus == 'uploaded' ) ) {
				unset( $videoIDs[$key] );
			}
		}

		// any remaining IDs in our list means they weren't found on YouTube
		foreach ( $videoIDs as $key => $value ) {
			$key = apply_filters( 'vb_delete_video', $key );
			if ( ! $key ) {
				continue;
			}
			wp_trash_post( $key );
			$this->vbp->info_message( sprintf( __( 'Schedule Checker: YouTube video %s not found. Sending post %s to trash.', 'autovid-profit' ), $value, $key ), 'updated', 'critical' );
		}
		return 0;
	}

	/**
	 * Creates batches of YouTube ids from save posts to check
	 */
	public function check_for_deleted_videos() {
		$this->vbp->info_message( sprintf( __( 'YouTube %s Start', 'autovid-profit' ), __FUNCTION__ ), 'updated', 'utility_funcs' );
		// get comma delimited videoIDs from post metadata
		$getMore = true;
		$offset = 0;
		$chunksize = apply_filters( 'vb_posts_chunksize', 1000 );

		while ( $getMore ) {
		$posts = $this->vbp->get_posts_by_site( 'YouTube', $offset );
		$num_results = count( $posts );
		$num=0;
		foreach ( $posts as $post ) {
			if ( ! $num ) {
				// use this to reset array after each batch
				$videoIDs = array();
			}
			$videoID = get_post_meta( $post->ID, 'VideoID', TRUE );
			if ( ! $videoID ) {     // mvb compatability check:
                                $videoID = get_post_meta( $post->ID, 'mvb_vid_code', TRUE );
                        }
			if ( ! $videoID ) {
				continue;
			}
			$videoIDs[$post->ID] = $videoID;
			if ( ++$num >= $this->batch_limit ) {
				$num = $this->check_for_deleted_batch( $videoIDs );
				$this->vbp->extend_execution_time();
			}
		} // end foreach posts
			$offset += $chunksize;
			$query_args['offset'] = $offset;
			if ( $num_results < $chunksize ) $getMore = false;
			$this->vbp->extend_execution_time();
		} // end getMore
		if ( $num ) {
			$this->check_for_deleted_batch( $videoIDs );	// still ids some left to check
		}
		$this->vbp->info_message( sprintf( __( 'YouTube %s End', 'autovid-profit' ), __FUNCTION__ ), 'updated', 'utility_funcs' );
	}

	/**
	 * Have to make a separate query for the video details and stats after a search query
	 * accepts a search result from query and/or a specific videoID
	 * https://developers.google.com/youtube/v3/docs/videos/list
	 */
	private function grab_videolist_details( $videos, $videoIDs='' ) {
		$this->vbp->info_message( __( 'Getting video details from results...', 'autovid-profit' ), 'updated', 'debug' );
		$base_url = $this->getApi( 'videos.list' );
		if ( ! empty( $videoIDs ) ) {
			$commaList = $videoIDs;
		}
		else if ( ! empty( $videos ) ) {
			$ids = array();
			foreach ( $videos as $video )  {
				if ( isset( $video->id->videoId ) ) {	// a search result
					$ids[] = $video->id->videoId;
				}
				else if ( isset( $video->id ) ) {	// a video result
					$ids[] = $video->id;
				}
			}
			$commaList = implode( ',', $ids );
		}
		else { // empty search query result
			return 0;
		}

        	$query_args = array(
                	'part'		=> 'snippet,contentDetails,statistics',
                	'id'		=> $commaList,
                	'key'		=> $this->apiKey
        	);
		$url = $base_url . '?' . http_build_query( $query_args );
		$data = $this->queryApi( $url );
		return isset( $data->items ) ? $data->items : 0;
	}

	private function get_best_thumbnail( $video ) {
		$img = array();
		$match = false;

		// check standard resolution thumbnails. all 3 should be set, but why take chances?

                // 120x90, ratio 4:3
                if ( isset( $video->snippet->thumbnails->default ) ) $img[] = $video->snippet->thumbnails->default->url;
                if ( $this->query_fields['qExtraParams2'] == 'default' ) $match = true;

                // 480x360, ratio 4:3
                if ( ! $match && isset( $video->snippet->thumbnails->high ) ) $img[] = $video->snippet->thumbnails->high->url;
                if ( $this->query_fields['qExtraParams2'] == 'high' ) $match = true;

                // 640x480, ratio 4:3
                if ( ! $match && isset( $video->snippet->thumbnails->standard ) ) $img[] = $video->snippet->thumbnails->standard->url;
                if ( $this->query_fields['qExtraParams2'] == 'standard' ) $match = true;

                // 320x180, ratio 16:9
                if ( ! $match && isset( $video->snippet->thumbnails->medium ) ) $img[] = $video->snippet->thumbnails->medium->url;
                if ( $this->query_fields['qExtraParams2'] == 'medium' ) $match = true;

                // 1280x720, ratio 16:9
                if ( ! $match && isset( $video->snippet->thumbnails->maxres ) ) $img[] = $video->snippet->thumbnails->maxres->url;

		$img = array_reverse( $img );

		$image = $this->vbp->verify_thumbnails( $img );

		return $image;
	}


	/**
	 * Extract video details into our generic videoInfo array
	 */
	private function get_video_info( $video ) {
		$videoInfo = array();

		// snippet info: ===================================================
		$videoInfo['orig_title'] = $videoInfo['title'] = isset( $video->snippet->title ) ? $video->snippet->title : '';
		$videoInfo['orig_desc'] = $videoInfo['desc'] = isset( $video->snippet->description ) ? $video->snippet->description : '';
		$videoInfo['channel'] = isset( $video->snippet->channelTitle ) ? $video->snippet->channelTitle : '';
		$videoInfo['channelID'] = isset( $video->snippet->channelId ) ? $video->snippet->channelId : '';
		$videoInfo['association'] = $videoInfo['channel'];
		$videoInfo['categoryID'] = isset( $video->snippet->categoryId ) ? $video->snippet->categoryId : 0;
		$videoInfo['videoID'] = isset( $video->id ) ? $video->id : 0;
		if ( isset( $video->snippet->playlistId ) ) {	// this is from a playlist
			$videoInfo['videoID'] = $video->snippet->resourceId->videoId;
		}
		$videoInfo['url'] = 'https://www.youtube.com/watch?v=' . $videoInfo['videoID'];

                // Make a separate call to YouTube oembed for embed code
                $url = "http://www.youtube.com/oembed?url=" . htmlentities( $videoInfo['url'] ) . "&format=json";
		$videoInfo['videoEmbed'] = $this->vbp->queryoEmbed( $url );

		$publishedAt = isset( $video->snippet->publishedAt ) ? $video->snippet->publishedAt : current_time( 'Y-m-d H:i:s' );
		$date = $this->vbp->getDateTime( $publishedAt, $videoInfo['url'] );
		$videoInfo['publishedAt'] = isset( $date ) ? $date->format( 'Y-m-d H:i:s' ) : $publishedAt;
		$videoInfo['publishedAt8601'] = isset( $date ) ? $date->format( 'c' ) : '';

		$videoInfo['sourceCategory'] = '';
		// get the YouTube video category?
		if ( isset( $video->snippet->playlistId ) ) {	// this is from a playlist
			$this->vbp->info_message( __( 'Notice: YouTube does not provide a category for Playlist items', 'autovid-profit' ), 'updated', 'video_import' );
		}
		else if ( TRUE == $this->query_fields['cAddSourceCategory'] && $videoInfo['categoryID'] ) {
			$categories = $this->grab_categories();
			foreach ( $categories as $cat ) {
				if ( $cat->id == $videoInfo['categoryID'] && isset( $cat->snippet->title ) ) {
					$videoInfo['sourceCategoryName'] = $cat->snippet->title;
					$videoInfo['sourceCategory'] = $this->vbp->create_the_category( $cat->snippet->title );
				}
			}
		}

		$videoInfo['img'] = $this->get_best_thumbnail( $video );

		// contentDetails info: =============================================
		$videoInfo['duration'] = '';
		if ( isset( $video->snippet->playlistId ) ) {	// this is from a playlist
			$this->vbp->info_message( __( 'Notice: YouTube does not provide a video duration for Playlist items', 'autovid-profit' ), 'updated', 'video_import' );
		}
		else if ( isset( $video->contentDetails->duration ) ) {
			// let's try to parse the given ISO 8601 time ( ie: PT1H11M11S )
			preg_match_all('|([0-9]+)([a-z])|Ui', $video->contentDetails->duration, $matches );
			if( isset( $matches[2] ) ){
				$videoInfo['duration'] = '';
				$hours=$minutes=$seconds=0;
				foreach( $matches[2] as $key => $unit ) {
					if ( $unit == 'H' ) $hours = $matches[1][$key];
					else if ( $unit == 'M' ) $minutes = $matches[1][$key];
					else if ( $unit == 'S' ) $seconds = $matches[1][$key];
				}
				$videoInfo['duration'] = $hours ? sprintf( "%02s:%02s:%02s", $hours, $minutes, $seconds ) : sprintf( "%02s:%02s", $minutes, $seconds );
				$this->vbp->info_message( sprintf( __( 'Converted duration %s to %s.', 'autovid-profit' ), $video->contentDetails->duration, $videoInfo['duration'] ), 'updated', 'debug' );
			}
		}

		// statistics info: ==================================================
		// not available on playlists :(
		if ( isset( $video->snippet->playlistId ) ) {	// this is from a playlist
			$this->vbp->info_message( __( 'Notice: YouTube does not provide statistics for Playlist items', 'autovid-profit' ), 'updated', 'video_import' );
		}
		$videoInfo['viewCount'] = isset( $video->statistics->viewCount ) ? $video->statistics->viewCount : 0;
		$videoInfo['likeCount'] = isset( $video->statistics->likeCount ) ? $video->statistics->likeCount : 0;
		$videoInfo['dislikeCount'] = isset( $video->statistics->dislikeCount ) ? $video->statistics->dislikeCount : 0;
		$videoInfo['favoriteCount'] = isset( $video->statistics->favoriteCount ) ? $video->statistics->favoriteCount : 0;
		$videoInfo['commentCount'] = isset( $video->statistics->commentCount ) ? $video->statistics->commentCount : 0;

		// external info: ====================================================
		$videoInfo['tags'] = '';
		// snippet tags only visible to YouTube video uploader so use this alternate method:
		if ( TRUE == $this->query_fields['pGrabTags'] ) {
			// 08/3/15: tags now included in YouTube data? this is new. check for it but keep fallback method too.
			if ( isset( $video->snippet->tags ) ) { 
				$videoInfo['tags'] = implode( ',', $video->snippet->tags );
			}
			else {
				$videoInfo['tags'] = $this->vbp->get_meta_tags( $videoInfo['url'] );
			}
		}

		$videoInfo['videoSource'] = 'YouTube';

		$videoInfo = apply_filters( 'get_video_info', $videoInfo );

		return $videoInfo;
	}

	/**
	 * Takes an array of video details to process and create posts
	 */
	private function save_videos( $videoDetails ) {

		if ( ! $videoDetails ) {
			return;
		}

		foreach ( $videoDetails as $video )  {

			$videoInfo = $this->get_video_info( $video );
			$this->vbp->info_message( sprintf( __( 'Checking YouTube video: [%s] at %s', 'autovid-profit' ), $videoInfo['title'], $videoInfo['url'] ), 'updated', 'video_import' );
			if ( $videoInfo['title'] == 'Deleted video' ) {
				$this->vbp->info_message( __( 'Deleted video detected. Skipping.', 'autovid-profit' ), 'updated', 'video_import' );
				continue;
			}

			// limited 'utf-8' language check until YouTube v3 API is better at language support
			if ( isset( $this->query_fields['qLanguage'] ) && stripos( $this->query_fields['qLanguage'], 'u8' ) !== FALSE ) {
				if ( ! $this->vbp->is_utf_8( $videoInfo['title'] ) ) {
					$this->vbp->info_message( sprintf( __( '%s is NOT utf-8 compliant. Skipping.', 'autovid-profit' ), $videoInfo['title'] ), 'updated', 'video_import' );
					continue;
				}
			}

			$postID = $this->vbp->save_the_video( $this->query_fields, $videoInfo );

			if ( ! $postID ) continue;

			if ( ! empty( $this->query_fields['qNumComments'] ) && $videoInfo['commentCount'] > 0 ) {
				$this->grab_comments( $videoInfo['videoID'], $postID );
			}

			$this->vbp->publish_the_video( $postID, $this->query_fields, $videoInfo );
		}
	}

	/**
	 * Will grab up to $this->batch_limit vids from a playlist at a time
	 */
	private function grab_playlist_batch( $url ) {
		$data = $this->queryApi( $url );
		$videoDetails = isset( $data->items ) ? $data->items : 0;
		$nextPage = isset( $data->nextPageToken ) ? $data->nextPageToken : 0;

		// playlistItems do not provide video category, duration or statistics
		// so grab all video ids from playlist and grab video list details instead. a bit more quota cost :(

		$videoIDs = array();
		foreach ( $videoDetails as $video ) {
			if ( isset( $video->contentDetails->videoId ) ) {
				$videoIDs[] = $video->contentDetails->videoId;
			}
		}
		$commaList = implode( ',', $videoIDs );
		$this->vbp->info_message( sprintf( __( 'Fetching playlist videoIDs: %s', 'autovid-profit' ), $commaList ), 'updated', 'video_import' );
		$videoDetails = $this->grab_videolist_details( '', $commaList );

		$this->save_videos( $videoDetails );

		return $nextPage;
	}

	/**
	 * Query for videos on a playlist
	 * https://developers.google.com/youtube/v3/docs/playlistItems/list
	 */
	private function grab_playlist() {
		$this->vbp->info_message( sprintf( __( 'Importing from playlist: %s', 'autovid-profit' ), $this->query_fields['qAssoc'] ), 'updated', 'video_import' );
		$totalVids = $this->query_fields['qNumVideos'] ? $this->query_fields['qNumVideos'] : -1;	// default grab all vids
        	$query_args = array(
                	'part'		=> 'contentDetails',
                	'playlistId'	=> $this->query_fields['qAssoc'],
			'maxResults'	=> $totalVids > $this->batch_limit || $totalVids < 0 ? $this->batch_limit : $totalVids,
                	'key'		=> $this->apiKey
        	);
		
		$query_args = apply_filters( 'youtube_playlist_query', $query_args, $this->query_fields );

		while ( $totalVids ) {
			$url = $this->getApi( 'playlistItems.list' ) . '?' . http_build_query( $query_args );
			$query_args['pageToken'] = $this->grab_playlist_batch( $url );
                        if ( ! $query_args['pageToken'] ) { // No videos found. No need to request more.
                                break;
			}
                        $totalVids -= $query_args['maxResults'];
                        $query_args['maxResults'] = $totalVids > $this->batch_limit || $totalVids < 0 ? $this->batch_limit : $totalVids;
                }
	}

	/**
	 * Grab a single video by the video ID stored in qAssoc.
	 */
	private function grab_single_video() {
		$videoDetails = $this->grab_videolist_details( '', $this->query_fields['qAssoc'] );
		$this->save_videos( $videoDetails );
	}

	/**
	 * Do a batch query search for up to $this->batch_limit vids at a time
	 */
	private function grab_videos_batch( $url ) {
		$data = $this->queryApi( $url );
		// initial search query has limited data, so make a separate query to get video details
		$videoDetails = ( isset( $data->items ) ) ? $this->grab_videolist_details( $data->items, '' ) : 0;
		$this->save_videos( $videoDetails );
		return ( isset( $data->nextPageToken ) ) ? $data->nextPageToken : 0;
	}

////////////////////////////////////////////////////////////////////////////////////////////

	/**
	 * Process the video query and break down into batches
	 * https://developers.google.com/youtube/v3/docs/search/list
	 */
	public function grab_videos() {

		$keyphrase = $this->query_fields['qKeyphrase'];
		$assoc = $this->query_fields['qAssoc'];
		$assocType = $this->query_fields['qAssocType'];
		$query_type = 'search.list';

                if ( empty( $keyphrase ) && ( empty( $assoc ) || empty( $assocType ) ) && $assocType != 'mostPopular' ) {
                        return $this->vbp->info_message( __( 'Error: No YouTube keyphrase or association not set properly.', 'autovid-profit' ), 'error', 'critical' );
                }

		if ( ! $assoc ) { // use keyphrase only
			$channelID = '';
		}
		else if ( $assocType == 'video' ) {
			return $this->grab_single_video();
		}
		else if ( $assocType == 'playlist' ) {
			return $this->grab_playlist();
		}
		else if ( $assocType == 'channel' ) {
			$channelID = $assoc;
		}
		else if ( $assocType == 'user' ) {
			$channelID = $this->grab_channel_id( $assoc );
			if ( ! $channelID ) {	# no such user/channel found - ABORT
				return 0;
			 }
		}
		else { // shouldn't happen
			return $this->vbp->info_message( sprintf( __( 'Error: association %s not recognized.', 'autovid-profit' ), $assocType ) );
		}

		$totalVids = $this->query_fields['qNumVideos'];
		
        	$query_args = array(
			'part' 			=> 'snippet',
			'maxResults' 		=> $totalVids > $this->batch_limit ? $this->batch_limit : $totalVids,
			'order'			=> ! empty( $this->query_fields['qOrderBy'] ) ? $this->query_fields['qOrderBy'] : 'date',
			'type' 			=> 'video',
			'videoDuration'		=> ! empty( $this->query_fields['qDuration'] ) ? $this->query_fields['qDuration'] : 'any',
			'videoDefinition'	=> ! empty( $this->query_fields['qDefinition'] ) ? $this->query_fields['qDefinition'] : 'any',
			'videoEmbeddable'       => 'TRUE',				// must set type to video
			'videoSyndicated'       => ! empty( $this->query_fields['qSyndication'] ) ? $this->query_fields['qSyndication'] : 'true',
			'safeSearch'       	=> ! empty( $this->query_fields['qSafeSearch'] ) ? $this->query_fields['qSafeSearch'] : 'moderate',
                	'key'			=> $this->apiKey
		);
		// optional args:
		if ( $channelID ) {
			$query_args['channelId'] = $channelID;
		}
		if ( $keyphrase ) {
			$query_args['q'] = $keyphrase;
		}
		if ( $assocType == 'mostPopular' ) {
			$query_args['chart'] = 'mostPopular';
			$query_type = 'videos.list';
		}
		if ( $this->query_fields['qCategory'] ) {
			$query_args['videoCategoryId'] = $this->query_fields['qCategory'];
		}
		if ( $this->query_fields['qRegionCode'] ) {
			$query_args['regionCode'] = $this->query_fields['qRegionCode'];
		}
		if ( $this->query_fields['qLanguage'] && $this->query_fields['qLanguage'] != 'u8' ) {
			$query_args['relevanceLanguage'] = $this->query_fields['qLanguage'];
		}
		if ( $this->query_fields['qPublishedAfter'] ) {
			$date = $this->vbp->getDateTime( $this->query_fields['qPublishedAfter'], $videoInfo['url'], false, false );
			if ( $date ) {
				$query_args['publishedAfter'] = $date->format( 'Y-m-d\TH:i:s.uP' );
			}
			else {
				$this->vbp->info_message( sprintf( __( 'Error: publishedAfter date %s not recognized.', 'autovid-profit' ), $query_args['publishedAfter'] ) );
			}
		}
		if ( $this->query_fields['qPublishedBefore'] ) {
			$date = $this->vbp->getDateTime( $this->query_fields['qPublishedBefore'], $videoInfo['url'], false, false );
			if ( $date ) {
				$query_args['publishedBefore'] = $date->format( 'Y-m-d\TH:i:s.uP' );
			}
			else {
				$this->vbp->info_message( sprintf( __( 'Error: publishedBefore date %s not recognized.', 'autovid-profit' ), $query_args['publishedBefore'] ) );
			}
		}

		$query_args = apply_filters( 'youtube_search_query', $query_args, $this->query_fields );

		while ( $totalVids > 0 ) {
			$url = $this->getApi( $query_type ) . '?' . http_build_query( $query_args );
			$query_args['pageToken'] = $this->grab_videos_batch( $url );
			if ( ! $query_args['pageToken'] ) { // No videos found. No need to request more.
				break;
			}
			$totalVids -= $query_args['maxResults'];
			$query_args['maxResults'] = $totalVids > $this->batch_limit ? $this->batch_limit : $totalVids;
		}
	}

	/**
	 * Gets the channel id from a user handle
	 */
	private function grab_channel_id( $user ) {
		$this->vbp->info_message( sprintf( __( 'Getting channelID for user %s.', 'autovid-profit' ), $user ), 'updated', 'debug' );
        	$query_args = array(
                	'forUsername'	=> $user,
                	'part'		=> 'id',
                	'key'		=> $this->apiKey
        	);
		$url = $this->getApi( 'channels.list' ) . '?' . http_build_query( $query_args );
		$data = $this->queryApi( $url );
        	if ( ! empty( $data->items[0] ) )  {
			return $data->items[0]->id;
		}
		return $this->vbp->info_message( sprintf( __( 'Error: No channelID returned by YouTube for user %s.', 'autovid-profit' ), $user ) );
	}

    } // END class Autovid_Profit_YouTube
} // END if ( ! class_exists( 'Autovid_Profit_YouTube' ) )
?>
